# Model result plot
# setwd("E:/Dropbox/Research/Manuscripts/Bivalve age model/tests/Gagan_coral_new")

require(tidyverse)
require(ggplot2)
require(gridExtra)

setwd("C:/Users//niels//Dropbox/Research/Manuscripts/Bivalve age model/tests/Case1_HR_results_new")
# setwd("E:/Dropbox/Research/Manuscripts/Bivalve age model/tests/Case1_HR_results_new")

# Import d18O model data
dOraw <- as.data.frame(read.csv("modelled_d18O_raw.csv", header = TRUE))
rownames(dOraw) <- dOraw$X # Add sample names as headers
dOraw$X <- NULL # Remove X column

d18Otidy <- gather(dOraw,
    "window",
    "d18O_mod",
    6:ncol(dOraw),
    factor_key = TRUE
    ) # Convert modelled d18O results to Tidy data for plotting

rho <- cor(d18Otidy$d18Oc,# Calculate Spearman's correlation coefficient
    d18Otidy$d18O_mod,
    use = "pairwise.complete.obs",
    method = "spearman"
    )
d18Odiff <- data.frame(diff = d18Otidy$d18O_mod - d18Otidy$d18Oc)
SDrep <- sd(d18Odiff$diff, na.rm = TRUE) # Calculate st error of estimate (stdev of reproducibility)

stats_table <- cbind(
    statistic = c("Spearman's correlation coefficient",
        "Standard error of the estimate"),
    value = c(round(rho, 2), round(SDrep, 3)),
    unit = c("-", "permille")
    )

# Create base plot of input data
d18Omodplot <- ggplot(data = dOraw, aes(D, d18Oc))

# Loop through windows to add results per window
for(window in seq(1, ncol(dOraw) - 5, 1)){
    d18Omodplot <- d18Omodplot +
        geom_line(aes_string(x = "D", y = paste("window.", window, sep = "")),
            col = "red",
            alpha = 0.2,
            size = 2)
}

# Add top layer of input data
d18Omodplot <- d18Omodplot +
    geom_point(size = 1, shape = 15) +
    geom_errorbar(aes(ymin = d18Oc - d18Oc_err, # Add error bars on data (1 SD)
        ymax = d18Oc + d18Oc_err),
        width = dOraw$D_err) +
    geom_errorbarh(aes(xmin = D - D_err,
        xmax = D + D_err),
        height = dOraw$d18Oc_err) +
    scale_x_continuous("Length along record (micrometers)",
        breaks = seq(floor(min(dOraw$D) / 10) * 10, ceiling(max(dOraw$D) / 10) * 10, 10),
        minor_breaks = seq(floor(min(dOraw$D) / 10) * 10, ceiling(max(dOraw$D) / 10) * 10, 5000)) +
    ggtitle("Simulations of d18O data (red) superimposed on input data (black)") +
    scale_y_continuous("d18Oc value (permille VPDB",
        breaks = seq(floor(min(dOraw$d18Oc)), ceiling(max(dOraw$d18Oc)), 0.5),
        minor_breaks = seq(floor(min(dOraw$d18Oc)), ceiling(max(dOraw$d18Oc)), 0.1)) +
    annotation_custom(tableGrob(stats_table),
        xmin = 0,
        xmax = 30,
        ymin = -3,
        ymax = -2)

# Export model result plot to PDF
pdf("d18O_model_result_plot.pdf", width = 10, height = 5)
plot(d18Omodplot)
dev.off()

# Histogram of d18O residuals
d18Omodhist <- ggplot(data = d18Odiff, aes(diff)) +
    geom_histogram(aes(y = ..density..),
        binwidth = 0.01,
        center = 0.005,
        na.rm = TRUE,
        col = "grey") +
    geom_vline(xintercept = 0,
        col = "red",
        size = 1) +
    geom_density(aes(y = ..density..),
        bw = "nrd0",
        kernel = "gaussian")

# Export histogram to PDF
pdf("d18O_residual_histogram.pdf", width = 5, height = 5)
plot(d18Omodhist)
dev.off()

# Import age model results
AMres <- as.data.frame(read.csv("Age_model_results.csv", header = TRUE))
rownames(AMres) <- AMres$X # Add sample names as headers
AMres$X <- NULL # Remove X column

# Import real age data
AMreal <- as.data.frame(read.csv("Real_data_Case1_HR2.csv", header = TRUE))
AMreal$Y <- AMreal$Y * 365 # Convert to days

# Calculate accuracy of model
AMdiff <- data.frame(D = AMres$D,
    Y = AMreal$Y,
    diff = AMres$mean.day - AMreal$Y
)
SDacc <- sd(AMdiff$diff, na.rm = TRUE) # Calculate st error of estimate (stdev of accuracy)

# Plot age model result plot
AMplot <- ggplot(data = AMres, aes(D, mean.day)) +
    geom_line(data = AMreal, aes(x = D, y = Y), col = "red", size = 1) +
    geom_ribbon(aes(ymin = mean.day - qt(0.95, N) * sd.day / sqrt(N),
        ymax = mean.day + qt(0.95, N) * sd.day / sqrt(N)),
        alpha = 0.5,
        col = "grey") +
    geom_line(size = 1) +
    geom_line(aes(x = D, y = mean.day + qt(0.95, N) * sd.day / sqrt(N)), col = "grey", size = 0.5) +
    geom_line(aes(x = D, y = mean.day - qt(0.95, N) * sd.day / sqrt(N)), col = "grey", size = 0.5) +
    scale_y_continuous("Age (days)",
        breaks = seq(floor(min(AMres$mean.day / 365)) * 365, ceiling(max(AMres$mean.day / 365)) * 365, 365),
        minor_breaks = seq(floor(min(AMres$mean.day / 365)) * 365, ceiling(max(AMres$mean.day / 365)) * 365, 365 / 12)) +
    scale_x_continuous("Length along record (micrometers)",
        breaks = seq(floor(min(AMres$D) / 10) * 10, ceiling(max(AMres$D) / 10) * 10, 10),
        minor_breaks = seq(floor(min(AMres$D) / 10) * 10, ceiling(max(AMres$D) / 10) * 10, 5000)) +
    ggtitle("Results of age model (Age in days vs. Depth in micrometers)")

# Export age model plot to PDF
pdf("Age_model_plot.pdf", width = 6, height = 6)
plot(AMplot)
dev.off()

# Plot accuracy vs depth plot
Accdepth <- ggplot(data = AMdiff, aes(D, diff)) +
    geom_col(aes(D, abs(diff), fill = diff)) +
    scale_fill_gradient2(low = "blue",
        high = "red",
        midpoint = 0) +
    scale_x_continuous("Length along record (mm)",
        breaks = seq(floor(min(AMres$D) / 10) * 10, ceiling(max(AMres$D) / 10) * 10, 10),
        minor_breaks = seq(floor(min(AMres$D) / 10) * 10, ceiling(max(AMres$D) / 10) * 10, 5000))

# Export accuracy vs. depth plot
pdf("Acc_depth_plot.pdf", width = 6, height = 2)
plot(Accdepth)
dev.off()

# Plot accuracy histogram
AMacchist <- ggplot(data = AMdiff, aes(diff)) +
    geom_histogram(aes(y = ..density..),
        binwidth = 10,
        center = 0,
        na.rm = TRUE,
        col = "lightgrey") +
    geom_vline(xintercept = 0,
        col = "red",
        size = 1) +
    geom_density(aes(y = ..density..),
        bw = "nrd0",
        kernel = "gaussian") +
    scale_x_continuous("offset in days",
        breaks = seq(floor(min(AMdiff$diff / 50)) * 50, ceiling(max(AMdiff$diff / 50)) * 50, 50),
        minor_breaks = seq(floor(min(AMdiff$diff / 50)) * 50, ceiling(max(AMdiff$diff / 50)) * 50, 10))

# Export accuracy histogram to PDF
pdf("age_accuracy_histogram.pdf", width = 5, height = 5)
plot(AMacchist)
dev.off()

# Bin accuracy by "months" of the year
breaks <- seq(0, 365, 365/12)
monthmean <- vector()
monthsd <- vector()
binmean <- vector()
for(i in 1:(length(breaks) - 1)){
    monthmean <- append(monthmean, mean(AMdiff[which(((AMdiff$Y %% 365) > breaks[i]) & ((AMdiff$Y %% 365) < breaks[i + 1])), 3]))
    monthsd <- append(monthsd, sd(AMdiff[which(((AMdiff$Y %% 365) > breaks[i]) & ((AMdiff$Y %% 365) < breaks[i + 1])), 3]))
    binmean <- append(binmean, mean(breaks[i], breaks[i + 1]))
}
AMdiff_bin <- data.frame(D = binmean,
    month.mean = monthmean,
    month.sd = monthsd)

# Plot accuracy vs. time of year
AccYD <- ggplot(data = AMdiff_bin, aes(D, month.mean)) +
    geom_point(aes(D, month.mean)) +
    geom_line() +
    geom_errorbar(aes(ymin = month.mean - month.sd,
        ymax = month.mean + month.sd,
        width = 10)) +
    ylab("offset from actual date (days)") +
    xlab("Day of the year") +
    geom_hline(yintercept = 0,
        col = "red",
        size = 1)

# Export monthly binned accuracy graph
pdf("monthly_accuracy.pdf", width = 5, height = 5)
plot(AccYD)
dev.off()