function [Omod,Rmod,Dmod,Omodsamp,Dmodsamp,JDmodsamp] = GrowthModel(Tamp,Tpha,Tpos,Gamp,Gpha,Gpos,Gskw,Osamp,Dsamp)
% SCRIPT CREATED: January 2017 (E. Judd)
% LAST CHANGED: September 2017 (E. Judd)

%FUNCTION TO GENERATE A TEMPERATURE/ISOTOPE AND GROWTH SINUSOID IN TIME
%SPACE. THIS CODE IS ASSOCIATED WITH THE GROWTH MODEL PRESENTED IN:
    % Judd, E.J., Wilkinson, B.H., and Ivany, L.C. (2017). The life and
    % time of clams: Derivation of intra-annual growth rates from
    % high-resolution oxygen isotope profiles. Palaeogeography,
    % Palaeoclimatology, Palaeoecology. 
    % https://doi.org/10.1016/j.palaeo.2017.09.034

%INPUTS:
    %Temp. sine parameters (amplitude, phase, position)
    %Growth sine parameters (amplitude, phase, postion, skew)
    %Osamp (year of isotope data)
    %Dsamp (same year of distance measurments)

%OUTPUTS
    %Omod: modeled isotope sinusoid for all 365 days of the yr
    %Rmod: modeled growth rate function for all 365 days of the yr
    %Dmod: distance values for all 365 days of the yr (cum sum of Rmod)
    %Omodsamp: oxygen isotope values for the modeled dataset (N = Osamp)
    %Dmodsamp: distance values for the modeled dataset (N = Dsamp)
    %JDmodsamp: Julian Date of each of the modeled isotope values

per = 365;
JD = linspace(1,365,365)';

%Initial temp sine
Omod = ((sin((JD-Tpha-(per/4))*pi/per*2)*Tamp/2)+Tpos)';

%Initial growth sine
for i = 1:length(JD)
    if mod(JD(i)+Gpha-(per/4),per) <= (per*Gskw/100)/2
        v = mod(JD(i)+Gpha-(per/4),per)/(2*(per*Gskw/100));
    elseif mod(JD(i)+Gpha-(per/4),per) <= (per*Gskw/100)/2+(per*(1-Gskw/100))
        v = (.25+(mod(JD(i)+Gpha-(per/4),per)-(per*Gskw/100)/2)/((per*(1-Gskw/100))*2));
    else
        v = (.75+(mod(JD(i)+Gpha-(per/4),per)-((per*Gskw/100)/2+(per*(1-Gskw/100))))/((per*Gskw/100)/2*4));
    end
    
    if (Gamp/2)*sin(2*pi*v)+Gpos < 0
        Rmodint(i) = 0;
    else
        Rmodint(i) = (Gamp/2)*sin(2*pi*v)+Gpos;
    end
end
Rmodint = Rmodint';

%Correct length
Dmodint = cumsum(Rmodint);
miol = min(Dsamp); %min observed length
maol = max(Dsamp); %max observed length
miml = min(Dmodint);  %min modeled length
maml = max(Dmodint);  %max modeled length
Dmod = (Dmodint*(maol-miol)/(maml-miml))-(miml*(maol-miol)/(maml-miml))+miol;
Rmod(1)=0;
Rmod(2:365)=diff(Dmod);
Rmod(1)=maol-miol-sum(Rmod);

%Extract Omodsamp
for i = 1:length(Dsamp)
    if i==1 & Rmod(2)== 0
        m = diff(Dmod); n = find(m>0);
        Dmodsamp(i) = Dmod(n(1));
        Omodsamp(i) = Omod(n(1));
        JDmodsamp(i) = JD(n(1));
    else
        [c index] = min(abs(Dmod-Dsamp(i)));
        Dmodsamp(i) = Dmod(index);
        Omodsamp(i) = Omod(index);
        JDmodsamp(i) = JD(index);
    end
    Omodsamp = Omodsamp';Dmodsamp = Dmodsamp';JDmodsamp = JDmodsamp';
end
end



